import numpy as np

import pyopencl as cl
import pyopencl.array

from pytential import bind, sym

def run_torus_refinement():
    ctx = cl.create_some_context()
    queue = cl.CommandQueue(ctx)

    target_order = 4
    source_ovsmp = 8
    qbx_order = 2

    # create mesh
    r_major = 1.0
    r_minor = 0.5
    n_major = 8
    n_minor = 4
    refine_rounds = 0

    from meshmode.mesh.generation import generate_torus
    mesh = generate_torus(r_major, r_minor,
        n_major=8, n_minor=4, order=target_order)

    if refine_rounds > 0:
        pass

    # create discretization
    from meshmode.discretization import Discretization
    from meshmode.discretization.poly_element import \
            InterpolatoryQuadratureSimplexGroupFactory
    discr = Discretization(ctx, mesh,
            InterpolatoryQuadratureSimplexGroupFactory(target_order))

    # create layer potential
    from pytential.qbx import QBXLayerPotentialSource
    qbx = QBXLayerPotentialSource(
            discr,
            fine_order=source_ovsmp * target_order,
            qbx_order=qbx_order,
            fmm_order=False,
            )

    # refine
    import pytential.qbx.refinement
    if hasattr(pytential.qbx.refinement, "refine_geometry_collection"):
        # NOTE: for `geometry-collection-caching` branch
        from pytential.qbx.refinement import refine_geometry_collection
        from pytential import GeometryCollection
        places = GeometryCollection(qbx)
        places = refine_geometry_collection(queue, places,
                refine_discr_stage=sym.QBX_SOURCE_STAGE1,
                maxiter=6,
                visualize=True)
    else:
        # NOTE: for `master` branch
        qbx, _ = qbx.with_refinement(
                maxiter=6,
                visualize=True)


if __name__ == "__main__":
    run_torus_refinement()
